@@ -8,13 +8,9 @@ ARCH=$2
 TMPDIR=$3
 
 if [ "$GOVERSION" = "gotip" ]; then
-    # TODO: remove this
-    if [ "$arch" != "amd64" ]; then
-        exit 0
-    fi
-    bootstrapver=$(curl https://golang.org/VERSION?m=text)
+    bootstrapver=$(curl https://go.dev/VERSION?m=text | head -1)
     cd $TMPDIR
-    curl -sSL "https://storage.googleapis.com/golang/$bootstrapver.darwin-$ARCH.tar.gz" | tar -xz
+    curl -sSL "https://go.dev/dl/$bootstrapver.darwin-$ARCH.tar.gz" | tar -xz
     cd -
     if [ -x $TMPDIR/go-tip ]; then
     	cd $TMPDIR/go-tip
@@ -29,10 +25,10 @@ if [ "$GOVERSION" = "gotip" ]; then
     cd -
 else
     echo Finding latest patch version for $GOVERSION
-    GOVERSION=$(python _scripts/latestver.py $GOVERSION)
+    GOVERSION=$(python3 _scripts/latestver.py $GOVERSION)
     echo Go $GOVERSION on $ARCH
     cd $TMPDIR
-    curl -sSL "https://storage.googleapis.com/golang/$GOVERSION.darwin-$ARCH.tar.gz" | tar -xz
+    curl -sSL "https://go.dev/dl/$GOVERSION.darwin-$ARCH.tar.gz" | tar -xz
     cd -
     export GOROOT="$TMPDIR/go"
 fi
@@ -46,4 +42,13 @@ export GOARCH="$ARCH"
 export PATH="$GOROOT/bin:$PATH"
 go version
 
+go install github.com/google/capslock/cmd/capslock@latest
+
+set +e
 make test
+x=$?
+if [ "$GOVERSION" = "gotip" ]; then
+	exit 0
+else
+	exit $x
+fi
