@@ -3,13 +3,12 @@ package main_test
 import (
 	"bufio"
 	"bytes"
+	"encoding/json"
 	"flag"
 	"fmt"
-	"go/ast"
-	"go/token"
-	"go/types"
 	"io"
 	"net"
+	"net/http"
 	"os"
 	"os/exec"
 	"os/user"
@@ -27,7 +26,6 @@ import (
 	"github.com/go-delve/delve/service/dap/daptest"
 	"github.com/go-delve/delve/service/rpc2"
 	godap "github.com/google/go-dap"
-	"golang.org/x/tools/go/packages"
 )
 
 var testBackend string
@@ -57,6 +55,7 @@ func assertNoError(err error, t testing.TB, s string) {
 }
 
 func TestBuild(t *testing.T) {
+	t.Parallel()
 	const listenAddr = "127.0.0.1:40573"
 
 	dlvbin := protest.GetDlvBinary(t)
@@ -127,7 +126,7 @@ func testOutput(t *testing.T, dlvbin, output string, delveCmds []string) (stdout
 
 	// Give delve some time to compile and write the binary.
 	foundIt := false
-	for wait := 0; wait < 30; wait++ {
+	for range 30 {
 		_, err = os.Stat(debugbin)
 		if err == nil {
 			foundIt = true
@@ -171,6 +170,7 @@ func testOutput(t *testing.T, dlvbin, output string, delveCmds []string) (stdout
 // TestOutput verifies that the debug executable is created in the correct path
 // and removed after exit.
 func TestOutput(t *testing.T) {
+	t.Parallel()
 	dlvbin := protest.GetDlvBinary(t)
 
 	for _, output := range []string{"__debug_bin", "myownname", filepath.Join(t.TempDir(), "absolute.path")} {
@@ -187,7 +187,7 @@ func TestOutput(t *testing.T) {
 // TestUnattendedBreakpoint tests whether dlv will print a message to stderr when the client that sends continue is disconnected
 // or not.
 func TestUnattendedBreakpoint(t *testing.T) {
-	const listenAddr = "127.0.0.1:40573"
+	const listenAddr = "127.0.0.1:40575"
 
 	fixturePath := filepath.Join(protest.FindFixturesDir(), "panic.go")
 	cmd := exec.Command(protest.GetDlvBinary(t), "debug", "--continue", "--headless", "--accept-multiclient", "--listen", listenAddr, fixturePath)
@@ -215,7 +215,8 @@ func TestUnattendedBreakpoint(t *testing.T) {
 
 // TestContinue verifies that the debugged executable starts immediately with --continue
 func TestContinue(t *testing.T) {
-	const listenAddr = "127.0.0.1:40573"
+	t.Parallel()
+	const listenAddr = "127.0.0.1:40576"
 
 	dlvbin := protest.GetDlvBinary(t)
 
@@ -247,7 +248,8 @@ func TestContinue(t *testing.T) {
 
 // TestRedirect verifies that redirecting stdin works
 func TestRedirect(t *testing.T) {
-	const listenAddr = "127.0.0.1:40573"
+	t.Parallel()
+	const listenAddr = "127.0.0.1:40574"
 
 	dlvbin := protest.GetDlvBinary(t)
 
@@ -328,16 +330,8 @@ func diffMaybe(t *testing.T, filename string, generated []byte) {
 // TestGeneratedDoc tests that the autogenerated documentation has been
 // updated.
 func TestGeneratedDoc(t *testing.T) {
-	if runtime.GOOS == "windows" && runtime.GOARCH == "arm64" {
-		//TODO(qmuntal): investigate further when the Windows ARM64 backend is more stable.
-		t.Skip("skipping test on Windows in CI")
-	}
-	if runtime.GOOS == "linux" && runtime.GOARCH == "ppc64le" {
-		//TODO(alexsaezm): finish CI integration
-		t.Skip("skipping test on Linux/PPC64LE in CI")
-	}
-	if goversion.VersionAfterOrEqual(runtime.Version(), 1, 24) {
-		t.Skip("disabled due to export format changes")
+	if !isLatestMinor(t) {
+		t.Skip("N/A")
 	}
 	// Checks gen-cli-docs.go
 	var generatedBuf bytes.Buffer
@@ -346,7 +340,7 @@ func TestGeneratedDoc(t *testing.T) {
 	checkAutogenDoc(t, "Documentation/cli/README.md", "_scripts/gen-cli-docs.go", generatedBuf.Bytes())
 
 	// Checks gen-usage-docs.go
-	if runtime.GOARCH == "ppc64le" {
+	if runtime.GOARCH != "ppc64le" {
 		tempDir := t.TempDir()
 		cmd := exec.Command("go", "run", "_scripts/gen-usage-docs.go", tempDir)
 		cmd.Dir = protest.ProjectRoot()
@@ -365,7 +359,12 @@ func TestGeneratedDoc(t *testing.T) {
 		a = append(a, args...)
 		cmd := exec.Command("go", a...)
 		cmd.Dir = protest.ProjectRoot()
-		out, err := cmd.CombinedOutput()
+		var b bytes.Buffer
+		cmd.Stderr = &b
+		out, err := cmd.Output()
+		if s := b.String(); s != "" {
+			t.Logf("stderr: %q", s)
+		}
 		if err != nil {
 			t.Fatalf("could not run script %v: %v (output: %q)", args, err, string(out))
 		}
@@ -373,17 +372,19 @@ func TestGeneratedDoc(t *testing.T) {
 	}
 
 	checkAutogenDoc(t, "Documentation/backend_test_health.md", "go run _scripts/gen-backend_test_health.go", runScript("_scripts/gen-backend_test_health.go", "-"))
-	checkAutogenDoc(t, "pkg/terminal/starbind/starlark_mapping.go", "'go generate' inside pkg/terminal/starbind", runScript("_scripts/gen-starlark-bindings.go", "go", "-"))
-	checkAutogenDoc(t, "Documentation/cli/starlark.md", "'go generate' inside pkg/terminal/starbind", runScript("_scripts/gen-starlark-bindings.go", "doc/dummy", "Documentation/cli/starlark.md"))
+	checkAutogenDoc(t, "pkg/terminal/starbind/starlark_mapping.go", "'go generate' inside pkg/terminal/starbind", runScript("github.com/go-delve/build-tools/cmd/gen-starlark-bindings@latest", "go", "-"))
+	checkAutogenDoc(t, "Documentation/cli/starlark.md", "'go generate' inside pkg/terminal/starbind", runScript("github.com/go-delve/build-tools/cmd/gen-starlark-bindings@latest", "doc/dummy", "Documentation/cli/starlark.md"))
 	checkAutogenDoc(t, "Documentation/faq.md", "'go run _scripts/gen-faq-toc.go Documentation/faq.md Documentation/faq.md'", runScript("_scripts/gen-faq-toc.go", "Documentation/faq.md", "-"))
-	checkAutogenDoc(t, "service/rpccommon/suitablemethods.go", "'go generate' inside service/rpccommon", runScript("_scripts/gen-suitablemethods.go", "-"))
+	checkAutogenDoc(t, "service/rpccommon/suitablemethods.go", "'go generate' inside service/rpccommon", runScript("github.com/go-delve/build-tools/cmd/gen-suitablemethods@latest", "-"))
 	if goversion.VersionAfterOrEqual(runtime.Version(), 1, 18) {
-		checkAutogenDoc(t, "_scripts/rtype-out.txt", "go run _scripts/rtype.go report _scripts/rtype-out.txt", runScript("_scripts/rtype.go", "report"))
-		runScript("_scripts/rtype.go", "check")
+		checkAutogenDoc(t, "_scripts/rtype-out.txt", "go run github.com/go-delve/build-tools/cmd/rtype@latest report _scripts/rtype-out.txt", runScript("github.com/go-delve/build-tools/cmd/rtype@latest", "report"))
+		runScript("github.com/go-delve/build-tools/cmd/rtype@latest", "check")
 	}
 }
 
 func TestExitInInit(t *testing.T) {
+	t.Parallel()
+
 	dlvbin := protest.GetDlvBinary(t)
 
 	buildtestdir := filepath.Join(protest.FindFixturesDir(), "buildtest")
@@ -399,190 +400,22 @@ func TestExitInInit(t *testing.T) {
 	}
 }
 
-func getMethods(pkg *types.Package, typename string) map[string]*types.Func {
-	r := make(map[string]*types.Func)
-	mset := types.NewMethodSet(types.NewPointer(pkg.Scope().Lookup(typename).Type()))
-	for i := 0; i < mset.Len(); i++ {
-		fn := mset.At(i).Obj().(*types.Func)
-		r[fn.Name()] = fn
-	}
-	return r
-}
-
-func publicMethodOf(decl ast.Decl, receiver string) *ast.FuncDecl {
-	fndecl, isfunc := decl.(*ast.FuncDecl)
-	if !isfunc {
-		return nil
-	}
-	if fndecl.Name.Name[0] >= 'a' && fndecl.Name.Name[0] <= 'z' {
-		return nil
-	}
-	if fndecl.Recv == nil || len(fndecl.Recv.List) != 1 {
-		return nil
-	}
-	starexpr, isstar := fndecl.Recv.List[0].Type.(*ast.StarExpr)
-	if !isstar {
-		return nil
-	}
-	identexpr, isident := starexpr.X.(*ast.Ident)
-	if !isident || identexpr.Name != receiver {
-		return nil
-	}
-	if fndecl.Body == nil {
-		return nil
-	}
-	return fndecl
-}
-
-func findCallCall(fndecl *ast.FuncDecl) *ast.CallExpr {
-	for _, stmt := range fndecl.Body.List {
-		var x ast.Expr = nil
-
-		switch s := stmt.(type) {
-		case *ast.AssignStmt:
-			if len(s.Rhs) == 1 {
-				x = s.Rhs[0]
-			}
-		case *ast.ReturnStmt:
-			if len(s.Results) == 1 {
-				x = s.Results[0]
-			}
-		case *ast.ExprStmt:
-			x = s.X
-		}
-
-		callx, iscall := x.(*ast.CallExpr)
-		if !iscall {
-			continue
-		}
-		fun, issel := callx.Fun.(*ast.SelectorExpr)
-		if !issel || (fun.Sel.Name != "call" && fun.Sel.Name != "callWhileDrainingEvents") {
-			continue
-		}
-		return callx
-	}
-	return nil
-}
-
-func qf(*types.Package) string {
-	return ""
-}
-
 func TestTypecheckRPC(t *testing.T) {
-	fset := &token.FileSet{}
-	cfg := &packages.Config{
-		Mode: packages.NeedSyntax | packages.NeedTypesInfo | packages.NeedName | packages.NeedCompiledGoFiles | packages.NeedTypes,
-		Fset: fset,
+	if !isLatestMinor(t) {
+		t.Skip("N/A")
 	}
-	pkgs, err := packages.Load(cfg, "github.com/go-delve/delve/service/rpc2")
+	cmd := exec.Command("go", "run", "github.com/go-delve/build-tools/cmd/typecheck-rpc@latest")
+	buf, err := cmd.CombinedOutput()
+	t.Logf("output: %s", string(buf))
 	if err != nil {
-		t.Fatal(err)
-	}
-	var clientAst *ast.File
-	var serverMethods map[string]*types.Func
-	var info *types.Info
-	packages.Visit(pkgs, func(pkg *packages.Package) bool {
-		if pkg.PkgPath != "github.com/go-delve/delve/service/rpc2" {
-			return true
-		}
-		t.Logf("package found: %v", pkg.PkgPath)
-		serverMethods = getMethods(pkg.Types, "RPCServer")
-		info = pkg.TypesInfo
-		for i := range pkg.Syntax {
-			t.Logf("file %q", pkg.CompiledGoFiles[i])
-			if strings.HasSuffix(pkg.CompiledGoFiles[i], string(os.PathSeparator)+"client.go") {
-				clientAst = pkg.Syntax[i]
-				break
-			}
-		}
-		return true
-	}, nil)
-
-	errcount := 0
-
-	for _, decl := range clientAst.Decls {
-		fndecl := publicMethodOf(decl, "RPCClient")
-		if fndecl == nil {
-			continue
-		}
-
-		switch fndecl.Name.Name {
-		case "Continue", "Rewind":
-			// wrappers over continueDir
-			continue
-		case "SetReturnValuesLoadConfig", "Disconnect", "SetEventsFn":
-			// support functions
-			continue
-		}
-
-		if fndecl.Name.Name == "Continue" || fndecl.Name.Name == "Rewind" || fndecl.Name.Name == "DirectionCongruentContinue" {
-			// using continueDir
-			continue
-		}
-
-		callx := findCallCall(fndecl)
-
-		if callx == nil {
-			t.Errorf("%s: could not find RPC call", fset.Position(fndecl.Pos()))
-			errcount++
-			continue
-		}
-
-		if len(callx.Args) != 3 {
-			t.Errorf("%s: wrong number of arguments for RPC call", fset.Position(callx.Pos()))
-			errcount++
-			continue
-		}
-
-		arg0, arg0islit := callx.Args[0].(*ast.BasicLit)
-		arg1 := callx.Args[1]
-		arg2 := callx.Args[2]
-		if !arg0islit || arg0.Kind != token.STRING {
-			continue
-		}
-		name, _ := strconv.Unquote(arg0.Value)
-		serverMethod := serverMethods[name]
-		if serverMethod == nil {
-			t.Errorf("%s: could not find RPC method %q", fset.Position(callx.Pos()), name)
-			errcount++
-			continue
-		}
-
-		params := serverMethod.Type().(*types.Signature).Params()
-
-		if a, e := info.TypeOf(arg1), params.At(0).Type(); !types.AssignableTo(a, e) {
-			t.Errorf("%s: wrong type of first argument %s, expected %s", fset.Position(callx.Pos()), types.TypeString(a, qf), types.TypeString(e, qf))
-			errcount++
-			continue
-		}
-
-		if !strings.HasSuffix(params.At(1).Type().String(), "/service.RPCCallback") {
-			if a, e := info.TypeOf(arg2), params.At(1).Type(); !types.AssignableTo(a, e) {
-				t.Errorf("%s: wrong type of second argument %s, expected %s", fset.Position(callx.Pos()), types.TypeString(a, qf), types.TypeString(e, qf))
-				errcount++
-				continue
-			}
-		}
-
-		if clit, ok := arg1.(*ast.CompositeLit); ok {
-			typ := params.At(0).Type()
-			st := typ.Underlying().(*types.Struct)
-			if len(clit.Elts) != st.NumFields() && types.TypeString(typ, qf) != "DebuggerCommand" {
-				t.Errorf("%s: wrong number of fields in first argument's literal %d, expected %d", fset.Position(callx.Pos()), len(clit.Elts), st.NumFields())
-				errcount++
-				continue
-			}
-		}
-	}
-
-	if errcount > 0 {
-		t.Errorf("%d errors", errcount)
+		t.Fatalf("error running typecheck-rpc: %v", err)
 	}
 }
 
 // TestDAPCmd verifies that a dap server can be started and shut down.
 func TestDAPCmd(t *testing.T) {
-	const listenAddr = "127.0.0.1:40575"
+	t.Parallel()
+	const listenAddr = "127.0.0.1:40580"
 
 	dlvbin := protest.GetDlvBinary(t)
 
@@ -632,7 +465,7 @@ func TestDAPCmd(t *testing.T) {
 
 func newDAPRemoteClient(t *testing.T, addr string, isDlvAttach bool, isMulti bool) *daptest.Client {
 	c := daptest.NewClient(addr)
-	c.AttachRequest(map[string]interface{}{"mode": "remote", "stopOnEntry": true})
+	c.AttachRequest(map[string]any{"mode": "remote", "stopOnEntry": true})
 	if isDlvAttach || isMulti {
 		c.ExpectCapabilitiesEventSupportTerminateDebuggee(t)
 	}
@@ -645,7 +478,8 @@ func newDAPRemoteClient(t *testing.T, addr string, isDlvAttach bool, isMulti boo
 }
 
 func TestRemoteDAPClient(t *testing.T) {
-	const listenAddr = "127.0.0.1:40576"
+	t.Parallel()
+	const listenAddr = "127.0.0.1:40577"
 
 	dlvbin := protest.GetDlvBinary(t)
 
@@ -698,7 +532,8 @@ func closeDAPRemoteMultiClient(t *testing.T, c *daptest.Client, expectStatus str
 }
 
 func TestRemoteDAPClientMulti(t *testing.T) {
-	const listenAddr = "127.0.0.1:40577"
+	t.Parallel()
+	const listenAddr = "127.0.0.1:40578"
 
 	dlvbin := protest.GetDlvBinary(t)
 
@@ -726,7 +561,7 @@ func TestRemoteDAPClientMulti(t *testing.T) {
 
 	// Client 0 connects but with the wrong attach request
 	dapclient0 := daptest.NewClient(listenAddr)
-	dapclient0.AttachRequest(map[string]interface{}{"mode": "local"})
+	dapclient0.AttachRequest(map[string]any{"mode": "local"})
 	dapclient0.ExpectErrorResponse(t)
 
 	// Client 1 connects and continues to main.main
@@ -749,7 +584,7 @@ func TestRemoteDAPClientMulti(t *testing.T) {
 
 	// Attach to exited processes is an error
 	dapclient3 := daptest.NewClient(listenAddr)
-	dapclient3.AttachRequest(map[string]interface{}{"mode": "remote", "stopOnEntry": true})
+	dapclient3.AttachRequest(map[string]any{"mode": "remote", "stopOnEntry": true})
 	dapclient3.ExpectErrorResponseWith(t, dap.FailedToAttach, `Process \d+ has exited with status 0`, true)
 	closeDAPRemoteMultiClient(t, dapclient3, "exited")
 
@@ -765,7 +600,8 @@ func TestRemoteDAPClientMulti(t *testing.T) {
 }
 
 func TestRemoteDAPClientAfterContinue(t *testing.T) {
-	const listenAddr = "127.0.0.1:40578"
+	t.Parallel()
+	const listenAddr = "127.0.0.1:40579"
 
 	dlvbin := protest.GetDlvBinary(t)
 
@@ -822,6 +658,7 @@ func TestRemoteDAPClientAfterContinue(t *testing.T) {
 
 // TestDAPCmdWithClient tests dlv dap --client-addr can be started and shut down.
 func TestDAPCmdWithClient(t *testing.T) {
+	t.Parallel()
 	listener, err := net.Listen("tcp", ":0")
 	if err != nil {
 		t.Fatalf("cannot setup listener required for testing: %v", err)
@@ -831,9 +668,6 @@ func TestDAPCmdWithClient(t *testing.T) {
 	dlvbin := protest.GetDlvBinary(t)
 
 	cmd := exec.Command(dlvbin, "dap", "--log-output=dap", "--log", "--client-addr", listener.Addr().String())
-	buf := &bytes.Buffer{}
-	cmd.Stdin = buf
-	cmd.Stdout = buf
 	assertNoError(cmd.Start(), t, "start dlv dap process with --client-addr flag")
 
 	// Wait for the connection.
@@ -857,13 +691,14 @@ func TestDAPCmdWithClient(t *testing.T) {
 	// Connection close should trigger dlv-reverse command's normal exit.
 	if err := cmd.Wait(); err != nil {
 		cmd.Process.Kill()
-		t.Fatalf("command failed: %v\n%s\n%v", err, buf.Bytes(), cmd.Process.Pid)
+		t.Fatalf("command failed: %v\n%v", err, cmd.Process.Pid)
 	}
 }
 
 // TestDAPCmdWithUnixClient tests dlv dap --client-addr can be started with unix domain socket and shut down.
 func TestDAPCmdWithUnixClient(t *testing.T) {
-	tmpdir := os.TempDir()
+	t.Parallel()
+	tmpdir := t.TempDir()
 	if tmpdir == "" {
 		return
 	}
@@ -878,9 +713,6 @@ func TestDAPCmdWithUnixClient(t *testing.T) {
 	dlvbin := protest.GetDlvBinary(t)
 
 	cmd := exec.Command(dlvbin, "dap", "--log-output=dap", "--log", "--client-addr=unix:"+listener.Addr().String())
-	buf := &bytes.Buffer{}
-	cmd.Stdin = buf
-	cmd.Stdout = buf
 	assertNoError(cmd.Start(), t, "start dlv dap process with --client-addr flag")
 
 	// Wait for the connection.
@@ -904,11 +736,12 @@ func TestDAPCmdWithUnixClient(t *testing.T) {
 	// Connection close should trigger dlv-reverse command's normal exit.
 	if err := cmd.Wait(); err != nil {
 		cmd.Process.Kill()
-		t.Fatalf("command failed: %v\n%s\n%v", err, buf.Bytes(), cmd.Process.Pid)
+		t.Fatalf("command failed: %v\n%v", err, cmd.Process.Pid)
 	}
 }
 
 func TestTrace(t *testing.T) {
+	t.Parallel()
 	dlvbin := protest.GetDlvBinary(t)
 
 	expected := []byte("> goroutine(1): main.foo(99, 9801)\n>> goroutine(1): main.foo => (9900)\n")
@@ -933,6 +766,7 @@ func TestTrace(t *testing.T) {
 }
 
 func TestTrace2(t *testing.T) {
+	t.Parallel()
 	dlvbin := protest.GetDlvBinary(t)
 
 	expected := []byte("> goroutine(1): main.callme(2)\n>> goroutine(1): main.callme => (4)\n")
@@ -957,6 +791,7 @@ func TestTrace2(t *testing.T) {
 }
 
 func TestTraceDirRecursion(t *testing.T) {
+	t.Parallel()
 	dlvbin := protest.GetDlvBinary(t)
 
 	expected := []byte("> goroutine(1):frame(1) main.A(5, 5)\n > goroutine(1):frame(2) main.A(4, 4)\n  > goroutine(1):frame(3) main.A(3, 3)\n   > goroutine(1):frame(4) main.A(2, 2)\n    > goroutine(1):frame(5) main.A(1, 1)\n    >> goroutine(1):frame(5) main.A => (1)\n   >> goroutine(1):frame(4) main.A => (2)\n  >> goroutine(1):frame(3) main.A => (6)\n >> goroutine(1):frame(2) main.A => (24)\n>> goroutine(1):frame(1) main.A => (120)\n")
@@ -990,6 +825,7 @@ func TestTraceDirRecursion(t *testing.T) {
 }
 
 func TestTraceMultipleGoroutines(t *testing.T) {
+	t.Parallel()
 	dlvbin := protest.GetDlvBinary(t)
 
 	// TODO(derekparker) this test has to be a bit vague to avoid flakiness.
@@ -1021,6 +857,7 @@ func TestTraceMultipleGoroutines(t *testing.T) {
 }
 
 func TestTracePid(t *testing.T) {
+	t.Parallel()
 	if runtime.GOOS == "linux" {
 		bs, _ := os.ReadFile("/proc/sys/kernel/yama/ptrace_scope")
 		if bs == nil || strings.TrimSpace(string(bs)) != "0" {
@@ -1041,10 +878,59 @@ func TestTracePid(t *testing.T) {
 	if targetCmd.Process == nil || targetCmd.Process.Pid == 0 {
 		t.Fatal("expected target process running")
 	}
+
 	defer targetCmd.Process.Kill()
 
 	// dlv attach the process by pid
 	cmd := exec.Command(dlvbin, "trace", "-p", strconv.Itoa(targetCmd.Process.Pid), "main.A")
+	defer cmd.Wait()
+
+	rdr, err := cmd.StderrPipe()
+	assertNoError(err, t, "stderr pipe")
+	defer rdr.Close()
+
+	assertNoError(cmd.Start(), t, "running trace")
+
+	output, err := io.ReadAll(rdr)
+	assertNoError(err, t, "ReadAll")
+
+	if !bytes.Contains(output, expected) {
+		t.Fatalf("expected:\n%s\ngot:\n%s", string(expected), string(output))
+	}
+}
+
+func TestTracePidNoKill(t *testing.T) {
+	if runtime.GOOS == "linux" {
+		bs, _ := os.ReadFile("/proc/sys/kernel/yama/ptrace_scope")
+		if bs == nil || strings.TrimSpace(string(bs)) != "0" {
+			t.Logf("can not run TestAttachDetach: %v\n", bs)
+			return
+		}
+	}
+
+	dlvbin := protest.GetDlvBinary(t)
+	expected := []byte("no breakpoints set")
+
+	// make process run
+	fix := protest.BuildFixture(t, "loopprog", 0)
+	targetCmd := exec.Command(fix.Path)
+	assertNoError(targetCmd.Start(), t, "execute loopprog")
+
+	if targetCmd.Process == nil || targetCmd.Process.Pid == 0 {
+		t.Fatal("expected target process running")
+	}
+
+	targetCmdDone := make(chan struct{})
+	go func() {
+		targetCmd.Process.Wait()
+		close(targetCmdDone)
+	}()
+
+	defer targetCmd.Process.Kill()
+
+	// attach to the process by pid and immediately exit
+	cmd := exec.Command(dlvbin, "trace", "-p", strconv.Itoa(targetCmd.Process.Pid), "invalidbreakpoint")
+
 	rdr, err := cmd.StderrPipe()
 	assertNoError(err, t, "stderr pipe")
 	defer rdr.Close()
@@ -1059,9 +945,17 @@ func TestTracePid(t *testing.T) {
 	}
 
 	cmd.Wait()
+
+	select {
+	case <-targetCmdDone:
+		t.Fatalf("expected process running after detach")
+	case <-time.After(200 * time.Millisecond):
+
+	}
 }
 
 func TestTraceBreakpointExists(t *testing.T) {
+	t.Parallel()
 	dlvbin := protest.GetDlvBinary(t)
 
 	fixtures := protest.FindFixturesDir()
@@ -1088,6 +982,7 @@ func TestTraceBreakpointExists(t *testing.T) {
 }
 
 func TestTracePrintStack(t *testing.T) {
+	t.Parallel()
 	dlvbin := protest.GetDlvBinary(t)
 
 	fixtures := protest.FindFixturesDir()
@@ -1110,6 +1005,7 @@ func TestTracePrintStack(t *testing.T) {
 }
 
 func TestTraceEBPF(t *testing.T) {
+	t.Parallel()
 	if os.Getenv("CI") == "true" {
 		t.Skip("cannot run test in CI, requires kernel compiled with btf support")
 	}
@@ -1149,6 +1045,7 @@ func TestTraceEBPF(t *testing.T) {
 }
 
 func TestTraceEBPF2(t *testing.T) {
+	t.Parallel()
 	if os.Getenv("CI") == "true" {
 		t.Skip("cannot run test in CI, requires kernel compiled with btf support")
 	}
@@ -1209,6 +1106,7 @@ func TestTraceEBPF2(t *testing.T) {
 }
 
 func TestTraceEBPF3(t *testing.T) {
+	t.Parallel()
 	if os.Getenv("CI") == "true" {
 		t.Skip("cannot run test in CI, requires kernel compiled with btf support")
 	}
@@ -1257,6 +1155,7 @@ func TestTraceEBPF3(t *testing.T) {
 }
 
 func TestTraceEBPF4(t *testing.T) {
+	t.Parallel()
 	if os.Getenv("CI") == "true" {
 		t.Skip("cannot run test in CI, requires kernel compiled with btf support")
 	}
@@ -1305,6 +1204,7 @@ func TestTraceEBPF4(t *testing.T) {
 }
 
 func TestDlvTestChdir(t *testing.T) {
+	t.Parallel()
 	dlvbin := protest.GetDlvBinary(t)
 
 	fixtures := protest.FindFixturesDir()
@@ -1337,6 +1237,7 @@ func TestDlvTestChdir(t *testing.T) {
 }
 
 func TestVersion(t *testing.T) {
+	t.Parallel()
 	dlvbin := protest.GetDlvBinary(t)
 
 	got, err := exec.Command(dlvbin, "version", "-v").CombinedOutput()
@@ -1351,6 +1252,7 @@ func TestVersion(t *testing.T) {
 }
 
 func TestStaticcheck(t *testing.T) {
+	t.Parallel()
 	_, err := exec.LookPath("staticcheck")
 	if err != nil {
 		t.Skip("staticcheck not installed")
@@ -1372,44 +1274,85 @@ func TestStaticcheck(t *testing.T) {
 	checkAutogenDoc(t, "_scripts/staticcheck-out.txt", fmt.Sprintf("staticcheck %s > _scripts/staticcheck-out.txt", strings.Join(args, " ")), out)
 }
 
+func TestCapsLock(t *testing.T) {
+	_, err := exec.LookPath("capslock")
+	if err != nil {
+		t.Skip("capslock not installed")
+	}
+	if !isLatestMinor(t) {
+		t.Skip("not on latest minor")
+	}
+
+	// Determine the expected output file based on current GOOS and GOARCH
+	goos := runtime.GOOS
+	goarch := runtime.GOARCH
+
+	// Normalize darwin to match our file naming
+	expectedFile := fmt.Sprintf("_scripts/capslock_%s_%s-output.txt", goos, goarch)
+
+	args := []string{"-packages", "./cmd/dlv"}
+	if goos == "linux" && goarch == "ppc64le" {
+		args = append([]string{"-buildtags", "exp.linuxppc64le"}, args...)
+	}
+
+	if goos == "linux" && goarch == "riscv64" {
+		args = append([]string{"-buildtags", "exp.linuxriscv64"}, args...)
+	}
+
+	cmd := exec.Command("capslock", args...)
+	cmd.Dir = protest.ProjectRoot()
+	cmd.Env = append(os.Environ(), fmt.Sprintf("GOOS=%s", goos), fmt.Sprintf("GOARCH=%s", goarch))
+	out, _ := cmd.CombinedOutput()
+
+	genCommand := "go run _scripts/gen-capslock-all.go"
+	checkAutogenDoc(t, expectedFile, genCommand, out)
+}
+
 func TestDefaultBinary(t *testing.T) {
+	t.Parallel()
 	// Check that when delve is run twice in the same directory simultaneously
 	// it will pick different default output binary paths.
 	dlvbin := protest.GetDlvBinary(t)
 	fixture := filepath.Join(protest.FindFixturesDir(), "testargs.go")
 
-	startOne := func() (io.WriteCloser, func() error, *bytes.Buffer) {
+	startOne := func() (io.WriteCloser, func() error, io.ReadCloser) {
 		cmd := exec.Command(dlvbin, "debug", "--allow-non-terminal-interactive=true", fixture, "--", "test")
 		stdin, _ := cmd.StdinPipe()
-		stdoutBuf := new(bytes.Buffer)
-		cmd.Stdout = stdoutBuf
+		stdout, err := cmd.StdoutPipe()
+		assertNoError(err, t, "cmd.StdoutPipe")
 
 		assertNoError(cmd.Start(), t, "dlv debug")
-		return stdin, cmd.Wait, stdoutBuf
+		return stdin, cmd.Wait, stdout
 	}
 
-	stdin1, wait1, stdoutBuf1 := startOne()
+	stdin1, wait1, stdout1 := startOne()
 	defer stdin1.Close()
+	defer stdout1.Close()
 
-	stdin2, wait2, stdoutBuf2 := startOne()
+	stdin2, wait2, stdout2 := startOne()
 	defer stdin2.Close()
+	defer stdout2.Close()
 
 	fmt.Fprintf(stdin1, "continue\nquit\n")
 	fmt.Fprintf(stdin2, "continue\nquit\n")
 
-	wait1()
-	wait2()
-
-	out1, out2 := stdoutBuf1.String(), stdoutBuf2.String()
-	t.Logf("%q", out1)
-	t.Logf("%q", out2)
-	if out1 == out2 {
+	out1, err := io.ReadAll(stdout1)
+	assertNoError(err, t, "io.ReadAll")
+	out2, err := io.ReadAll(stdout2)
+	assertNoError(err, t, "io.ReadAll")
+	t.Logf("%q", string(out1))
+	t.Logf("%q", string(out2))
+	if bytes.Equal(out1, out2) {
 		t.Errorf("outputs match")
 	}
+
+	wait1()
+	wait2()
 }
 
 func TestUnixDomainSocket(t *testing.T) {
-	tmpdir := os.TempDir()
+	t.Parallel()
+	tmpdir := t.TempDir()
 	if tmpdir == "" {
 		return
 	}
@@ -1461,6 +1404,7 @@ func TestUnixDomainSocket(t *testing.T) {
 }
 
 func TestDeadcodeEliminated(t *testing.T) {
+	t.Parallel()
 	dlvbin := protest.GetDlvBinary(t)
 	buf, err := exec.Command("go", "tool", "nm", dlvbin).CombinedOutput()
 	if err != nil {
@@ -1470,3 +1414,43 @@ func TestDeadcodeEliminated(t *testing.T) {
 		t.Fatal("output of go tool nm contains MethodByName")
 	}
 }
+
+func isLatestMinor(t *testing.T) bool {
+	t.Helper()
+	resp, err := http.Get("https://go.dev/dl/?mode=json&include=all")
+	if err != nil {
+		t.Logf("Error fetching versions: %v\n", err)
+		return false
+	}
+	defer resp.Body.Close()
+
+	var data []struct {
+		Version string `json:"version"`
+	}
+
+	if err := json.NewDecoder(resp.Body).Decode(&data); err != nil {
+		t.Logf("Error decoding JSON: %v\n", err)
+		return false
+	}
+
+	maxMinor := 0
+	for _, item := range data {
+		ver, ok := goversion.Parse(item.Version)
+		if !ok {
+			continue
+		}
+		if ver.Major != 1 {
+			t.Fatalf("unsupported major version found: %s", item.Version)
+		}
+		if ver.Minor > maxMinor {
+			maxMinor = ver.Minor
+		}
+	}
+
+	ver, ok := goversion.Parse(runtime.Version())
+	if !ok {
+		return false
+	}
+
+	return ver.Minor == maxMinor
+}
