@@ -3,11 +3,14 @@ package native
 import (
 	"fmt"
 	"os"
+	"strings"
 	"syscall"
+	"unicode/utf16"
 	"unsafe"
 
 	sys "golang.org/x/sys/windows"
 
+	"github.com/go-delve/delve/pkg/logflags"
 	"github.com/go-delve/delve/pkg/proc"
 	"github.com/go-delve/delve/pkg/proc/internal/ebpf"
 )
@@ -23,17 +26,17 @@ type osProcessDetails struct {
 func (os *osProcessDetails) Close() {}
 
 // Launch creates and begins debugging a new process.
-func Launch(cmd []string, wd string, flags proc.LaunchFlags, _ []string, _ string, redirects [3]string) (*proc.Target, error) {
+func Launch(cmd []string, wd string, flags proc.LaunchFlags, _ []string, _ string, stdinPath string, stdoutOR proc.OutputRedirect, stderrOR proc.OutputRedirect) (*proc.TargetGroup, error) {
 	argv0Go := cmd[0]
 
 	env := proc.DisableAsyncPreemptEnv()
 
-	stdin, stdout, stderr, closefn, err := openRedirects(redirects, true)
+	stdin, stdout, stderr, closefn, err := openRedirects(stdinPath, stdoutOR, stderrOR, true)
 	if err != nil {
 		return nil, err
 	}
 
-	creationFlags := uint32(_DEBUG_ONLY_THIS_PROCESS)
+	creationFlags := uint32(_DEBUG_PROCESS)
 	if flags&proc.LaunchForeground == 0 {
 		creationFlags |= syscall.CREATE_NEW_PROCESS_GROUP
 	}
@@ -62,37 +65,37 @@ func Launch(cmd []string, wd string, flags proc.LaunchFlags, _ []string, _ strin
 
 	tgt, err := dbp.initialize(argv0Go, []string{})
 	if err != nil {
-		dbp.Detach(true)
+		detachWithoutGroup(dbp, true)
 		return nil, err
 	}
 	return tgt, nil
 }
 
-func initialize(dbp *nativeProcess) error {
+func initialize(dbp *nativeProcess) (string, error) {
+	// we need a fake procgrp to call waitForDebugEvent
+	procgrp := &processGroup{procs: []*nativeProcess{dbp}}
 	// It should not actually be possible for the
 	// call to waitForDebugEvent to fail, since Windows
 	// will always fire a CREATE_PROCESS_DEBUG_EVENT event
 	// immediately after launching under DEBUG_ONLY_THIS_PROCESS.
 	// Attaching with DebugActiveProcess has similar effect.
 	var err error
-	var tid, exitCode int
 	dbp.execPtraceFunc(func() {
-		tid, exitCode, err = dbp.waitForDebugEvent(waitBlocking)
+		_, err = procgrp.waitForDebugEvent(waitBlocking)
 	})
 	if err != nil {
-		return err
-	}
-	if tid == 0 {
-		dbp.postExit()
-		return proc.ErrProcessExited{Pid: dbp.pid, Status: exitCode}
+		return "", err
 	}
+
+	cmdline := getCmdLine(dbp.os.hProcess)
+
 	// Suspend all threads so that the call to _ContinueDebugEvent will
 	// not resume the target.
 	for _, thread := range dbp.threads {
 		if !thread.os.dbgUiRemoteBreakIn {
 			_, err := _SuspendThread(thread.os.hThread)
 			if err != nil {
-				return err
+				return "", err
 			}
 		}
 	}
@@ -100,10 +103,10 @@ func initialize(dbp *nativeProcess) error {
 	dbp.execPtraceFunc(func() {
 		err = _ContinueDebugEvent(uint32(dbp.pid), uint32(dbp.os.breakThread), _DBG_CONTINUE)
 	})
-	return err
+	return cmdline, err
 }
 
-// findExePath searches for process pid, and returns its executable path.
+// findExePath searches for process pid, and returns its executable path
 func findExePath(pid int) (string, error) {
 	// Original code suggested different approach (see below).
 	// Maybe it could be useful in the future.
@@ -140,7 +143,7 @@ func findExePath(pid int) (string, error) {
 var debugPrivilegeRequested = false
 
 // Attach to an existing process with the given PID.
-func Attach(pid int, _ []string) (*proc.Target, error) {
+func Attach(pid int, waitFor *proc.WaitFor, _ []string) (*proc.TargetGroup, error) {
 	var aperr error
 	if !debugPrivilegeRequested {
 		debugPrivilegeRequested = true
@@ -151,6 +154,14 @@ func Attach(pid int, _ []string) (*proc.Target, error) {
 		aperr = acquireDebugPrivilege()
 	}
 
+	if waitFor.Valid() {
+		var err error
+		pid, err = WaitFor(waitFor)
+		if err != nil {
+			return nil, err
+		}
+	}
+
 	dbp := newProcess(pid)
 	var err error
 	dbp.execPtraceFunc(func() {
@@ -169,7 +180,7 @@ func Attach(pid int, _ []string) (*proc.Target, error) {
 	}
 	tgt, err := dbp.initialize(exepath, []string{})
 	if err != nil {
-		dbp.Detach(true)
+		detachWithoutGroup(dbp, true)
 		return nil, err
 	}
 	return tgt, nil
@@ -209,9 +220,46 @@ func acquireDebugPrivilege() error {
 	return nil
 }
 
+func waitForSearchProcess(pfx string, seen map[int]struct{}) (int, error) {
+	log := logflags.DebuggerLogger()
+	handle, err := sys.CreateToolhelp32Snapshot(sys.TH32CS_SNAPPROCESS, 0)
+	if err != nil {
+		return 0, fmt.Errorf("could not get process list: %v", err)
+	}
+	defer sys.CloseHandle(handle)
+
+	var entry sys.ProcessEntry32
+	entry.Size = uint32(unsafe.Sizeof(entry))
+	err = sys.Process32First(handle, &entry)
+	if err != nil {
+		return 0, fmt.Errorf("could not get process list: %v", err)
+	}
+
+	for err = sys.Process32First(handle, &entry); err == nil; err = sys.Process32Next(handle, &entry) {
+		if _, isseen := seen[int(entry.ProcessID)]; isseen {
+			continue
+		}
+		seen[int(entry.ProcessID)] = struct{}{}
+
+		hProcess, err := sys.OpenProcess(sys.PROCESS_QUERY_INFORMATION|sys.PROCESS_VM_READ, false, entry.ProcessID)
+		if err != nil {
+			continue
+		}
+		cmdline := getCmdLine(syscall.Handle(hProcess))
+		sys.CloseHandle(hProcess)
+
+		log.Debugf("waitfor: new process %q", cmdline)
+		if strings.HasPrefix(cmdline, pfx) {
+			return int(entry.ProcessID), nil
+		}
+	}
+
+	return 0, nil
+}
+
 // kill kills the process.
-func (dbp *nativeProcess) kill() error {
-	if dbp.exited {
+func (procgrp *processGroup) kill(dbp *nativeProcess) error {
+	if ok, _ := dbp.Valid(); !ok {
 		return nil
 	}
 
@@ -227,7 +275,7 @@ func (dbp *nativeProcess) kill() error {
 	_ = syscall.TerminateProcess(dbp.os.hProcess, 1)
 
 	dbp.execPtraceFunc(func() {
-		dbp.waitForDebugEvent(waitBlocking | waitDontHandleExceptions)
+		procgrp.waitForDebugEvent(waitBlocking | waitDontHandleExceptions)
 	})
 
 	p.Wait()
@@ -294,9 +342,8 @@ const (
 
 const _MS_VC_EXCEPTION = 0x406D1388 // part of VisualC protocol to set thread names
 
-func (dbp *nativeProcess) waitForDebugEvent(flags waitForDebugEventFlags) (threadID, exitCode int, err error) {
+func (procgrp *processGroup) waitForDebugEvent(flags waitForDebugEventFlags) (threadID int, err error) {
 	var debugEvent _DEBUG_EVENT
-	shouldExit := false
 	for {
 		continueStatus := uint32(_DBG_CONTINUE)
 		var milliseconds uint32 = 0
@@ -306,7 +353,7 @@ func (dbp *nativeProcess) waitForDebugEvent(flags waitForDebugEventFlags) (threa
 		// Wait for a debug event...
 		err := _WaitForDebugEvent(&debugEvent, milliseconds)
 		if err != nil {
-			return 0, 0, err
+			return 0, err
 		}
 
 		// ... handle each event kind ...
@@ -318,26 +365,64 @@ func (dbp *nativeProcess) waitForDebugEvent(flags waitForDebugEventFlags) (threa
 			if hFile != 0 && hFile != syscall.InvalidHandle {
 				err = syscall.CloseHandle(hFile)
 				if err != nil {
-					return 0, 0, err
+					return 0, err
 				}
 			}
+			var dbp *nativeProcess
+			if procgrp.addTarget == nil {
+				// This is a fake process group and waitForDebugEvent has been called
+				// just after attach/launch, finish configuring the root process.
+				dbp = procgrp.procs[0]
+				if debugEvent.ProcessId != uint32(dbp.pid) {
+					// When launching a new process while we are in detached state also
+					// creates a conhost.exe child process. This is that process and we
+					// need to detach from it.
+					// See issue #3864.
+					err := _DebugActiveProcessStop(debugEvent.ProcessId)
+					if err != nil {
+						return 0, err
+					}
+					continue
+				}
+			} else {
+				// Add new child process
+				dbp = newChildProcess(procgrp.procs[0], int(debugEvent.ProcessId))
+			}
+
 			dbp.os.entryPoint = uint64(debugInfo.BaseOfImage)
 			dbp.os.hProcess = debugInfo.Process
 			_, err = dbp.addThread(debugInfo.Thread, int(debugEvent.ThreadId), false,
 				flags&waitSuspendNewThreads != 0, debugInfo.StartAddress == dbgUiRemoteBreakin.Addr())
 			if err != nil {
-				return 0, 0, err
+				return 0, err
+			}
+
+			if procgrp.addTarget != nil {
+				exe, err := findExePath(dbp.pid)
+				if err != nil {
+					return 0, err
+				}
+				tgt, err := procgrp.add(dbp, dbp.pid, dbp.memthread, exe, proc.StopLaunched, getCmdLine(dbp.os.hProcess))
+				if err != nil {
+					// TODO: Handle proc.ErrBadBinaryInfo
+					return 0, err
+				}
+				if tgt == nil {
+					continue
+				}
 			}
 			break
 		case _CREATE_THREAD_DEBUG_EVENT:
 			debugInfo := (*_CREATE_THREAD_DEBUG_INFO)(unionPtr)
+			dbp := procgrp.procForPid(int(debugEvent.ProcessId))
 			_, err = dbp.addThread(debugInfo.Thread, int(debugEvent.ThreadId), false,
 				flags&waitSuspendNewThreads != 0, debugInfo.StartAddress == dbgUiRemoteBreakin.Addr())
 			if err != nil {
-				return 0, 0, err
+				return 0, err
 			}
 			break
 		case _EXIT_THREAD_DEBUG_EVENT:
+			dbp := procgrp.procForPid(int(debugEvent.ProcessId))
 			delete(dbp.threads, int(debugEvent.ThreadId))
 			break
 		case _OUTPUT_DEBUG_STRING_EVENT:
@@ -349,7 +434,7 @@ func (dbp *nativeProcess) waitForDebugEvent(flags waitForDebugEventFlags) (threa
 			if hFile != 0 && hFile != syscall.InvalidHandle {
 				err = syscall.CloseHandle(hFile)
 				if err != nil {
-					return 0, 0, err
+					return 0, err
 				}
 			}
 			break
@@ -364,6 +449,7 @@ func (dbp *nativeProcess) waitForDebugEvent(flags waitForDebugEventFlags) (threa
 			}
 			exception := (*_EXCEPTION_DEBUG_INFO)(unionPtr)
 			tid := int(debugEvent.ThreadId)
+			dbp := procgrp.procForPid(int(debugEvent.ProcessId))
 
 			switch code := exception.ExceptionRecord.ExceptionCode; code {
 			case _EXCEPTION_BREAKPOINT:
@@ -393,13 +479,13 @@ func (dbp *nativeProcess) waitForDebugEvent(flags waitForDebugEventFlags) (threa
 					if th := dbp.threads[tid]; th != nil {
 						th.os.setbp = true
 					}
-					return tid, 0, nil
+					return tid, nil
 				} else {
 					continueStatus = _DBG_CONTINUE
 				}
 			case _EXCEPTION_SINGLE_STEP:
 				dbp.os.breakThread = tid
-				return tid, 0, nil
+				return tid, nil
 			case _MS_VC_EXCEPTION:
 				// This exception is sent to set the thread name in VisualC, we should
 				// mask it or it might crash the program.
@@ -409,80 +495,90 @@ func (dbp *nativeProcess) waitForDebugEvent(flags waitForDebugEventFlags) (threa
 			}
 		case _EXIT_PROCESS_DEBUG_EVENT:
 			debugInfo := (*_EXIT_PROCESS_DEBUG_INFO)(unionPtr)
-			exitCode = int(debugInfo.ExitCode)
-			shouldExit = true
+			dbp := procgrp.procForPid(int(debugEvent.ProcessId))
+			dbp.postExit()
+			if procgrp.numValid() == 0 {
+				err = _ContinueDebugEvent(debugEvent.ProcessId, debugEvent.ThreadId, continueStatus)
+				if err != nil {
+					return 0, err
+				}
+				return 0, proc.ErrProcessExited{Pid: dbp.pid, Status: int(debugInfo.ExitCode)}
+			}
 		default:
-			return 0, 0, fmt.Errorf("unknown debug event code: %d", debugEvent.DebugEventCode)
+			return 0, fmt.Errorf("unknown debug event code: %d", debugEvent.DebugEventCode)
 		}
 
 		// .. and then continue unless we received an event that indicated we should break into debugger.
 		err = _ContinueDebugEvent(debugEvent.ProcessId, debugEvent.ThreadId, continueStatus)
 		if err != nil {
-			return 0, 0, err
-		}
-
-		if shouldExit {
-			return 0, exitCode, nil
+			return 0, err
 		}
 	}
 }
 
-func (dbp *nativeProcess) trapWait(pid int) (*nativeThread, error) {
+func trapWait(procgrp *processGroup, pid int) (*nativeThread, error) {
 	var err error
-	var tid, exitCode int
-	dbp.execPtraceFunc(func() {
-		tid, exitCode, err = dbp.waitForDebugEvent(waitBlocking)
+	var tid int
+	procgrp.procs[0].execPtraceFunc(func() {
+		tid, err = procgrp.waitForDebugEvent(waitBlocking)
 	})
 	if err != nil {
 		return nil, err
 	}
-	if tid == 0 {
-		dbp.postExit()
-		return nil, proc.ErrProcessExited{Pid: dbp.pid, Status: exitCode}
-	}
-	th := dbp.threads[tid]
+	th := procgrp.procForThread(tid).threads[tid]
 	return th, nil
 }
 
-func (dbp *nativeProcess) wait(pid, options int) (int, *sys.WaitStatus, error) {
-	return 0, nil, fmt.Errorf("not implemented: wait")
-}
-
 func (dbp *nativeProcess) exitGuard(err error) error {
 	return err
 }
 
-func (dbp *nativeProcess) resume() error {
-	for _, thread := range dbp.threads {
-		if thread.CurrentBreakpoint.Breakpoint != nil {
-			if err := thread.StepInstruction(); err != nil {
-				return err
+func (procgrp *processGroup) resume() error {
+	for _, dbp := range procgrp.procs {
+		if valid, _ := dbp.Valid(); !valid {
+			continue
+		}
+		for _, thread := range dbp.threads {
+			if thread.CurrentBreakpoint.Breakpoint != nil {
+				if err := procgrp.stepInstruction(thread); err != nil {
+					return err
+				}
+				thread.CurrentBreakpoint.Clear()
 			}
-			thread.CurrentBreakpoint.Clear()
 		}
 	}
 
-	for _, thread := range dbp.threads {
-		_, err := _ResumeThread(thread.os.hThread)
-		if err != nil {
-			return err
+	for _, dbp := range procgrp.procs {
+		if valid, _ := dbp.Valid(); !valid {
+			continue
 		}
+		for _, thread := range dbp.threads {
+			_, err := _ResumeThread(thread.os.hThread)
+			if err != nil {
+				return err
+			}
+		}
+		dbp.os.running = true
 	}
-	dbp.os.running = true
 
 	return nil
 }
 
 // stop stops all running threads threads and sets breakpoints
-func (dbp *nativeProcess) stop(cctx *proc.ContinueOnceContext, trapthread *nativeThread) (*nativeThread, error) {
-	if dbp.exited {
-		return nil, proc.ErrProcessExited{Pid: dbp.pid}
+func (procgrp *processGroup) stop(cctx *proc.ContinueOnceContext, trapthread *nativeThread) (*nativeThread, error) {
+	if procgrp.numValid() == 0 {
+		return nil, proc.ErrProcessExited{Pid: procgrp.procs[0].pid}
 	}
 
-	dbp.os.running = false
-	for _, th := range dbp.threads {
-		th.os.setbp = false
+	trapproc := procgrp.procForThread(trapthread.ID)
+	trapproc.os.running = false
+
+	for _, dbp := range procgrp.procs {
+		for _, th := range dbp.threads {
+			th.os.setbp = false
+		}
 	}
+
 	trapthread.os.setbp = true
 
 	// While the debug event that stopped the target was being propagated
@@ -500,28 +596,21 @@ func (dbp *nativeProcess) stop(cctx *proc.ContinueOnceContext, trapthread *nativ
 		return nil, err
 	}
 
-	context := newContext()
-
-	for _, thread := range dbp.threads {
-		thread.os.delayErr = nil
-		if !thread.os.dbgUiRemoteBreakIn {
-			// Wait before reporting the error, the thread could be removed when we
-			// call waitForDebugEvent in the next loop.
-			_, thread.os.delayErr = _SuspendThread(thread.os.hThread)
-			if thread.os.delayErr == nil {
-				// This call will block until the thread has stopped.
-				_ = thread.getContext(context)
-			}
+	for _, dbp := range procgrp.procs {
+		if valid, _ := dbp.Valid(); !valid {
+			continue
 		}
+		dbp.suspendAllThreads()
 	}
 
+	lastEventProc := trapproc
 	for {
 		var err error
 		var tid int
-		dbp.execPtraceFunc(func() {
-			err = _ContinueDebugEvent(uint32(dbp.pid), uint32(dbp.os.breakThread), _DBG_CONTINUE)
+		lastEventProc.execPtraceFunc(func() {
+			err = _ContinueDebugEvent(uint32(lastEventProc.pid), uint32(lastEventProc.os.breakThread), _DBG_CONTINUE)
 			if err == nil {
-				tid, _, _ = dbp.waitForDebugEvent(waitSuspendNewThreads)
+				tid, _ = procgrp.waitForDebugEvent(waitSuspendNewThreads)
 			}
 		})
 		if err != nil {
@@ -530,16 +619,18 @@ func (dbp *nativeProcess) stop(cctx *proc.ContinueOnceContext, trapthread *nativ
 		if tid == 0 {
 			break
 		}
+		dbp := procgrp.procForThread(tid)
 		err = dbp.threads[tid].SetCurrentBreakpoint(true)
 		if err != nil {
 			return nil, err
 		}
+		lastEventProc = dbp
 	}
 
 	// Check if trapthread still exist, if the process is dying it could have
 	// been removed while we were stopping the other threads.
 	trapthreadFound := false
-	for _, thread := range dbp.threads {
+	for _, thread := range trapproc.threads {
 		if thread.ID == trapthread.ID {
 			trapthreadFound = true
 		}
@@ -554,7 +645,7 @@ func (dbp *nativeProcess) stop(cctx *proc.ContinueOnceContext, trapthread *nativ
 		wasDbgUiRemoteBreakIn := trapthread.os.dbgUiRemoteBreakIn
 		// trapthread exited during stop, pick another one
 		trapthread = nil
-		for _, thread := range dbp.threads {
+		for _, thread := range trapproc.threads {
 			if thread.CurrentBreakpoint.Breakpoint != nil && thread.os.delayErr == nil {
 				trapthread = thread
 				break
@@ -563,7 +654,7 @@ func (dbp *nativeProcess) stop(cctx *proc.ContinueOnceContext, trapthread *nativ
 		if trapthread == nil && wasDbgUiRemoteBreakIn {
 			// If this was triggered by a manual stop request we should stop
 			// regardless, pick a thread.
-			for _, thread := range dbp.threads {
+			for _, thread := range trapproc.threads {
 				return thread, nil
 			}
 		}
@@ -572,6 +663,35 @@ func (dbp *nativeProcess) stop(cctx *proc.ContinueOnceContext, trapthread *nativ
 	return trapthread, nil
 }
 
+func (dbp *nativeProcess) suspendAllThreads() {
+	context := newContext()
+	for _, thread := range dbp.threads {
+		thread.os.delayErr = nil
+		if !thread.os.dbgUiRemoteBreakIn {
+			// Wait before reporting the error, the thread could be removed when we
+			// call waitForDebugEvent in the next loop.
+			_, thread.os.delayErr = _SuspendThread(thread.os.hThread)
+			if thread.os.delayErr == nil {
+				// This call will block until the thread has stopped.
+				_ = thread.getContext(context)
+			}
+		}
+	}
+}
+
+func (procgrp *processGroup) detachChild(dbp *nativeProcess) error {
+	for _, thread := range dbp.threads {
+		_, err := _ResumeThread(thread.os.hThread)
+		if err != nil {
+			return err
+		}
+	}
+	err := _DebugActiveProcessStop(uint32(dbp.pid))
+	dbp.detached.Store(true)
+	dbp.postExit()
+	return err
+}
+
 func (dbp *nativeProcess) detach(kill bool) error {
 	if !kill {
 		//TODO(aarzilli): when debug.Target exist Detach should be moved to
@@ -602,6 +722,135 @@ func (dbp *nativeProcess) GetBufferedTracepoints() []ebpf.RawUProbeParams {
 	return nil
 }
 
+type _PROCESS_BASIC_INFORMATION struct {
+	ExitStatus                   sys.NTStatus
+	PebBaseAddress               uintptr
+	AffinityMask                 uintptr
+	BasePriority                 int32
+	UniqueProcessId              uintptr
+	InheritedFromUniqueProcessId uintptr
+}
+
+type _PEB struct {
+	reserved1              [2]byte
+	BeingDebugged          byte
+	BitField               byte
+	reserved3              uintptr
+	ImageBaseAddress       uintptr
+	Ldr                    uintptr
+	ProcessParameters      uintptr
+	reserved4              [3]uintptr
+	AtlThunkSListPtr       uintptr
+	reserved5              uintptr
+	reserved6              uint32
+	reserved7              uintptr
+	reserved8              uint32
+	AtlThunkSListPtr32     uint32
+	reserved9              [45]uintptr
+	reserved10             [96]byte
+	PostProcessInitRoutine uintptr
+	reserved11             [128]byte
+	reserved12             [1]uintptr
+	SessionId              uint32
+}
+
+type _RTL_USER_PROCESS_PARAMETERS struct {
+	MaximumLength, Length uint32
+
+	Flags, DebugFlags uint32
+
+	ConsoleHandle                                sys.Handle
+	ConsoleFlags                                 uint32
+	StandardInput, StandardOutput, StandardError sys.Handle
+
+	CurrentDirectory struct {
+		DosPath _NTUnicodeString
+		Handle  sys.Handle
+	}
+
+	DllPath       _NTUnicodeString
+	ImagePathName _NTUnicodeString
+	CommandLine   _NTUnicodeString
+	Environment   unsafe.Pointer
+
+	StartingX, StartingY, CountX, CountY, CountCharsX, CountCharsY, FillAttribute uint32
+
+	WindowFlags, ShowWindowFlags                     uint32
+	WindowTitle, DesktopInfo, ShellInfo, RuntimeData _NTUnicodeString
+	CurrentDirectories                               [32]struct {
+		Flags     uint16
+		Length    uint16
+		TimeStamp uint32
+		DosPath   _NTString
+	}
+
+	EnvironmentSize, EnvironmentVersion uintptr
+
+	PackageDependencyData uintptr
+	ProcessGroupId        uint32
+	LoaderThreads         uint32
+
+	RedirectionDllName               _NTUnicodeString
+	HeapPartitionName                _NTUnicodeString
+	DefaultThreadpoolCpuSetMasks     uintptr
+	DefaultThreadpoolCpuSetMaskCount uint32
+}
+
+type _NTString struct {
+	Length        uint16
+	MaximumLength uint16
+	Buffer        uintptr
+}
+
+type _NTUnicodeString struct {
+	Length        uint16
+	MaximumLength uint16
+	Buffer        uintptr
+}
+
+func getCmdLine(hProcess syscall.Handle) string {
+	logger := logflags.DebuggerLogger()
+	var info _PROCESS_BASIC_INFORMATION
+	err := sys.NtQueryInformationProcess(sys.Handle(hProcess), sys.ProcessBasicInformation, unsafe.Pointer(&info), uint32(unsafe.Sizeof(info)), nil)
+	if err != nil {
+		logger.Errorf("NtQueryInformationProcess: %v", err)
+		return ""
+	}
+	var peb _PEB
+	err = _ReadProcessMemory(hProcess, info.PebBaseAddress, (*byte)(unsafe.Pointer(&peb)), unsafe.Sizeof(peb), nil)
+	if err != nil {
+		logger.Errorf("Reading PEB: %v", err)
+		return ""
+	}
+	var upp _RTL_USER_PROCESS_PARAMETERS
+	err = _ReadProcessMemory(hProcess, peb.ProcessParameters, (*byte)(unsafe.Pointer(&upp)), unsafe.Sizeof(upp), nil)
+	if err != nil {
+		logger.Errorf("Reading ProcessParameters: %v", err)
+		return ""
+	}
+	if upp.CommandLine.Length%2 != 0 {
+		logger.Errorf("CommandLine length not a multiple of 2")
+		return ""
+	}
+	buf := make([]byte, upp.CommandLine.Length)
+	err = _ReadProcessMemory(hProcess, upp.CommandLine.Buffer, &buf[0], uintptr(len(buf)), nil)
+	if err != nil {
+		logger.Errorf("Reading CommandLine: %v", err)
+		return ""
+	}
+	utf16buf := make([]uint16, len(buf)/2)
+	for i := 0; i < len(buf); i += 2 {
+		utf16buf[i/2] = uint16(buf[i+1])<<8 + uint16(buf[i])
+	}
+	return string(utf16.Decode(utf16buf))
+}
+
+// FollowExec enables (or disables) follow exec mode
+func (dbp *nativeProcess) FollowExec(v bool) error {
+	dbp.followExec = v
+	return nil
+}
+
 func killProcess(pid int) error {
 	p, err := os.FindProcess(pid)
 	if err != nil {
