@@ -24,6 +24,7 @@ type TargetGroup struct {
 	Selected          *Target
 	followExecEnabled bool
 	followExecRegex   *regexp.Regexp
+	eventsFn          func(*Event)
 
 	RecordingManipulation
 	recman RecordingManipulationInternal
@@ -106,12 +107,38 @@ func Restart(grp, oldgrp *TargetGroup, discard func(*LogicalBreakpoint, error))
 func (grp *TargetGroup) addTarget(p ProcessInternal, pid int, currentThread Thread, path string, stopReason StopReason, cmdline string) (*Target, error) {
 	logger := logflags.DebuggerLogger()
 	if len(grp.targets) > 0 {
+		matchesPattern := grp.followExecRegex == nil || grp.followExecRegex.MatchString(cmdline)
 		if !grp.followExecEnabled {
 			logger.Debugf("Detaching from child target (follow-exec disabled) %d %q", pid, cmdline)
-			return nil, nil
-		}
-		if grp.followExecRegex != nil && !grp.followExecRegex.MatchString(cmdline) {
+		} else if !matchesPattern {
 			logger.Debugf("Detaching from child target (follow-exec regex not matched) %d %q", pid, cmdline)
+		}
+
+		// Notify listeners that a child process was spawned.
+		//
+		// We do this regardless of whether the process is debuggable and
+		// regardless of whether follow-exec is enabled; when run by an IDE in
+		// DAP mode, we want to give the IDE a chance to handle the spawned
+		// process.
+		//
+		// Defer the call so that - if the process _can_ be debugged - listeners
+		// are notified after the new target is set up.
+		willFollow := grp.followExecEnabled && matchesPattern
+		if grp.Selected != nil {
+			if fn := grp.Selected.BinInfo().eventsFn; fn != nil {
+				defer fn(&Event{
+					Kind: EventProcessSpawned,
+					ProcessSpawnedEventDetails: &ProcessSpawnedEventDetails{
+						PID:        pid,
+						ThreadID:   currentThread.ThreadID(),
+						Cmdline:    cmdline,
+						WillFollow: willFollow,
+					},
+				})
+			}
+		}
+
+		if !willFollow {
 			return nil, nil
 		}
 	}
@@ -132,16 +159,32 @@ func (grp *TargetGroup) addTarget(p ProcessInternal, pid int, currentThread Thre
 	if grp.Selected == nil {
 		grp.Selected = t
 	}
+	t.BinInfo().eventsFn = grp.eventsFn
 	t.Breakpoints().Logical = grp.LogicalBreakpoints
 	for _, lbp := range grp.LogicalBreakpoints {
 		if lbp.LogicalID < 0 {
 			continue
 		}
+		var wasSuspended bool
+		if grp.eventsFn != nil {
+			wasSuspended = lbp.isSuspendedOnGroup(grp)
+		}
 		err := enableBreakpointOnTarget(t, lbp)
 		if err != nil {
 			logger.Debugf("could not enable breakpoint %d on new target %d: %v", lbp.LogicalID, t.Pid(), err)
 		} else {
 			logger.Debugf("breakpoint %d enabled on new target %d: %v", lbp.LogicalID, t.Pid(), err)
+			if grp.eventsFn != nil {
+				isSuspended := lbp.isSuspendedOnTarget(t)
+				if wasSuspended && !isSuspended {
+					grp.eventsFn(&Event{
+						Kind: EventBreakpointMaterialized,
+						BreakpointMaterializedEventDetails: &BreakpointMaterializedEventDetails{
+							Breakpoint: lbp,
+						},
+					})
+				}
+			}
 		}
 	}
 	grp.targets = append(grp.targets, t)
@@ -537,6 +580,31 @@ func (grp *TargetGroup) FollowExecEnabled() bool {
 	return grp.followExecEnabled
 }
 
+// SetEventsFn sets a function that is called to communicate events
+// happening while the target process is running.
+func (grp *TargetGroup) SetEventsFn(eventsFn func(*Event)) {
+	grp.eventsFn = eventsFn
+	it := ValidTargets{Group: grp}
+	for it.Next() {
+		it.BinInfo().eventsFn = eventsFn
+	}
+}
+
+// CancelDownloads cancels ongoing downloads, if any.
+func (grp *TargetGroup) CancelDownloads() bool {
+	r := false
+	it := ValidTargets{Group: grp}
+	for it.Next() {
+		it.BinInfo().cancelDownloadsMu.Lock()
+		if it.BinInfo().cancelDownloads != nil {
+			it.BinInfo().cancelDownloads()
+			r = true
+		}
+		it.BinInfo().cancelDownloadsMu.Unlock()
+	}
+	return r
+}
+
 // ValidTargets iterates through all valid targets in Group.
 type ValidTargets struct {
 	*Target
@@ -564,3 +632,39 @@ func (it *ValidTargets) Reset() {
 	it.Target = nil
 	it.start = 0
 }
+
+// Event is an event that happened during execution of the debugged program.
+type Event struct {
+	Kind EventKind
+	*BinaryInfoDownloadEventDetails
+	*BreakpointMaterializedEventDetails
+	*ProcessSpawnedEventDetails
+}
+
+type EventKind uint8
+
+const (
+	EventResumed EventKind = iota
+	EventStopped
+	EventBinaryInfoDownload
+	EventBreakpointMaterialized
+	EventProcessSpawned
+)
+
+// BinaryInfoDownloadEventDetails describes the details of a BinaryInfoDownloadEvent
+type BinaryInfoDownloadEventDetails struct {
+	ImagePath, Progress string
+}
+
+// BreakpointMaterializedEventDetails describes the details of a BreakpointMaterializedEvent
+type BreakpointMaterializedEventDetails struct {
+	Breakpoint *LogicalBreakpoint
+}
+
+// ProcessSpawnedEventDetails describes the details of a ProcessSpawnedEvent
+type ProcessSpawnedEventDetails struct {
+	PID        int
+	ThreadID   int
+	Cmdline    string
+	WillFollow bool
+}
