@@ -10,6 +10,7 @@ import (
 	"path/filepath"
 	"regexp"
 	"runtime"
+	"slices"
 	"strings"
 	"sync"
 	"testing"
@@ -22,12 +23,6 @@ var EnableRace = flag.Bool("racetarget", false, "Enables race detector on inferi
 
 var runningWithFixtures bool
 
-var ldFlags string
-
-func init() {
-	ldFlags = os.Getenv("CGO_LDFLAGS")
-}
-
 // Fixture is a test binary.
 type Fixture struct {
 	// Name is the short name of the fixture.
@@ -38,6 +33,8 @@ type Fixture struct {
 	Source string
 	// BuildDir is the directory where the build command was run.
 	BuildDir string
+	// buildDone is closed when the fixture is built
+	buildDone <-chan struct{}
 }
 
 // FixtureKey holds the name and builds flags used for a test fixture.
@@ -47,17 +44,25 @@ type fixtureKey struct {
 }
 
 // Fixtures is a map of fixtureKey{ Fixture.Name, buildFlags } to Fixture.
-var fixtures = make(map[fixtureKey]Fixture)
+var fixtures = make(map[fixtureKey]*Fixture)
+var fixturesmu sync.Mutex
 
-// PathsToRemove is a list of files and directories to remove after running all the tests
-var PathsToRemove []string
+// pathsToRemove is a list of files and directories to remove after running all the tests
+var pathsToRemove []string
+var pathmu sync.Mutex
+
+func AddPathToRemove(path string) {
+	pathmu.Lock()
+	defer pathmu.Unlock()
+	pathsToRemove = append(pathsToRemove, path)
+}
 
 // FindFixturesDir will search for the directory holding all test fixtures
 // beginning with the current directory and searching up 10 directories.
 func FindFixturesDir() string {
 	parent := ".."
 	fixturesDir := "_fixtures"
-	for depth := 0; depth < 10; depth++ {
+	for range 10 {
 		if _, err := os.Stat(fixturesDir); err == nil {
 			break
 		}
@@ -102,9 +107,16 @@ func BuildFixture(t testing.TB, name string, flags BuildFlags) Fixture {
 		panic("RunTestsWithFixtures not called")
 	}
 	fk := fixtureKey{name, flags}
+	fixturesmu.Lock()
 	if f, ok := fixtures[fk]; ok {
-		return f
+		fixturesmu.Unlock()
+		<-f.buildDone
+		return *f
 	}
+	buildDone := make(chan struct{})
+	fixture := Fixture{Name: name, buildDone: buildDone}
+	fixtures[fk] = &fixture
+	fixturesmu.Unlock()
 
 	if flags&EnableCGOOptimization == 0 {
 		if os.Getenv("CI") == "" || os.Getenv("CGO_CFLAGS") == "" {
@@ -187,6 +199,20 @@ func BuildFixture(t testing.TB, name string, flags BuildFlags) Fixture {
 		os.Exit(1)
 	}
 
+	source, _ := filepath.Abs(path)
+	source = filepath.ToSlash(source)
+	sympath, err := filepath.EvalSymlinks(source)
+	if err == nil {
+		source = strings.ReplaceAll(sympath, "\\", "/")
+	}
+
+	absdir, _ := filepath.Abs(dir)
+
+	fixture.Path = tmpfile
+	fixture.Source = source
+	fixture.BuildDir = absdir
+	close(buildDone)
+
 	if flags&EnableDWZCompression != 0 {
 		cmd := exec.Command("dwz", tmpfile)
 		if out, err := cmd.CombinedOutput(); err != nil {
@@ -200,19 +226,7 @@ func BuildFixture(t testing.TB, name string, flags BuildFlags) Fixture {
 		}
 	}
 
-	source, _ := filepath.Abs(path)
-	source = filepath.ToSlash(source)
-	sympath, err := filepath.EvalSymlinks(source)
-	if err == nil {
-		source = strings.ReplaceAll(sympath, "\\", "/")
-	}
-
-	absdir, _ := filepath.Abs(dir)
-
-	fixture := Fixture{Name: name, Path: tmpfile, Source: source, BuildDir: absdir}
-
-	fixtures[fk] = fixture
-	return fixtures[fk]
+	return fixture
 }
 
 // RunTestsWithFixtures sets the flag runningWithFixtures to compile fixtures on demand and runs tests with m.Run().
@@ -229,7 +243,7 @@ func RunTestsWithFixtures(m *testing.M) {
 		os.Remove(f.Path)
 	}
 
-	for _, p := range PathsToRemove {
+	for _, p := range pathsToRemove {
 		fi, err := os.Stat(p)
 		if err != nil {
 			panic(err)
@@ -322,7 +336,9 @@ func MustSupportFunctionCalls(t *testing.T, testBackend string) {
 		t.Skip(fmt.Errorf("%s does not support FunctionCall for now", runtime.GOARCH))
 	}
 	if runtime.GOARCH == "loong64" {
-		t.Skip(fmt.Errorf("%s does not support FunctionCall for now", runtime.GOARCH))
+		if !goversion.VersionAfterOrEqual(runtime.Version(), 1, 24) {
+			t.Skip("function calls not supported on loong64 with Go < 1.24")
+		}
 	}
 	if runtime.GOARCH == "arm64" {
 		if !goversion.VersionAfterOrEqual(runtime.Version(), 1, 19) || runtime.GOOS == "windows" {
@@ -431,15 +447,9 @@ func ProjectRoot() string {
 }
 
 func GetDlvBinary(t *testing.T) string {
-	// In case this was set in the environment
-	// from getDlvBinEBPF lets clear it here, so
-	// we can ensure we don't get build errors
-	// depending on the test ordering.
-	t.Setenv("CGO_LDFLAGS", ldFlags)
+	t.Helper()
+
 	var tags []string
-	if runtime.GOOS == "windows" && runtime.GOARCH == "arm64" {
-		tags = []string{"-tags=exp.winarm64"}
-	}
 	if runtime.GOOS == "linux" && runtime.GOARCH == "ppc64le" {
 		tags = []string{"-tags=exp.linuxppc64le"}
 	}
@@ -453,20 +463,40 @@ func GetDlvBinary(t *testing.T) string {
 }
 
 func GetDlvBinaryEBPF(t *testing.T) string {
+	t.Helper()
+
 	return getDlvBinInternal(t, "-tags", "ebpf")
 }
 
+// Fixtures is a map of fixtureKey{ Fixture.Name, buildFlags } to Fixture.
+var dlvbincache = make(map[string]string)
+var dlvbinmu sync.Mutex
+
 func getDlvBinInternal(t *testing.T, goflags ...string) string {
-	dlvbin := filepath.Join(t.TempDir(), "dlv.exe")
+	dlvbinmu.Lock()
+	defer dlvbinmu.Unlock()
+
+	// Parse GOFLAGS and filter out empty strings
+	goenvflags := os.Getenv("GOFLAGS")
+	if goenvflags != "" {
+		goflags = slices.Concat(goflags, strings.Split(os.Getenv("GOFLAGS"), " "))
+	}
+
+	strargs := strings.Join(goflags, "")
+	if path, ok := dlvbincache[strargs]; ok {
+		return path
+	}
+
+	dlvbin := TempFile("dlv.exe")
+
+	dlvbincache[strargs] = dlvbin
+	AddPathToRemove(dlvbin)
+
 	args := append([]string{"build", "-o", dlvbin}, goflags...)
 	args = append(args, "github.com/go-delve/delve/cmd/dlv")
-
-	wd, _ := os.Getwd()
-	fmt.Printf("at %s %s\n", wd, goflags)
-
 	out, err := exec.Command("go", args...).CombinedOutput()
 	if err != nil {
-		t.Fatalf("go build -o %v github.com/go-delve/delve/cmd/dlv: %v\n%s", dlvbin, err, string(out))
+		t.Fatalf("go %s: %v\n%s", strings.Join(args, " "), err, string(out))
 	}
 
 	return dlvbin
