@@ -114,7 +114,7 @@ func TestVariableEvaluation(t *testing.T) {
 	testcases := []struct {
 		name        string
 		st          reflect.Kind
-		value       interface{}
+		value       any
 		length, cap int64
 		childrenlen int
 	}{
@@ -815,7 +815,7 @@ func getEvalExpressionTestCases() []varTest {
 		{"i2 << f1", false, "", "", "", errors.New("shift count type float64, must be unsigned integer")},
 		{"i2 << -1", false, "", "", "", errors.New("shift count must not be negative")},
 		{"*(i2 + i3)", false, "", "", "", errors.New("expression \"(i2 + i3)\" (int) can not be dereferenced")},
-		{"i2.member", false, "", "", "", errors.New("i2 (type int) is not a struct")},
+		{"i2.member", false, "", "", "", errors.New("i2 (type int) has no member member")},
 		{"fmt.Println(\"hello\")", false, "", "", "", errors.New("function calls not allowed without using 'call'")},
 		{"*nil", false, "", "", "", errors.New("nil can not be dereferenced")},
 		{"!nil", false, "", "", "", errors.New("operator ! can not be applied to \"nil\"")},
@@ -958,6 +958,11 @@ func getEvalExpressionTestCases() []varTest {
 		testcases = append(testcases, varTest{`**(**runtime.hmap)(uintptr(&m1))`, false, `…`, `…`, "runtime.hmap", nil})
 	}
 
+	if goversion.VersionAfterOrEqualRev(runtime.Version(), 1, 25, 2) {
+		testcases = append(testcases, varTest{"iface7", true, "interface {}(main.OnlyUsedInInterface) {s: \"test\"}", "interface {}(main.OnlyUsedInInterface) {s: \"test\"}", "interface {}", nil})
+
+	}
+
 	return testcases
 }
 
@@ -1255,7 +1260,7 @@ func TestIssue1075(t *testing.T) {
 	withTestProcess("clientdo", t, func(p *proc.Target, grp *proc.TargetGroup, fixture protest.Fixture) {
 		setFunctionBreakpoint(p, t, "net/http.(*Client).Do")
 		assertNoError(grp.Continue(), t, "Continue()")
-		for i := 0; i < 10; i++ {
+		for i := range 10 {
 			scope, err := proc.GoroutineScope(p, p.CurrentThread())
 			assertNoError(err, t, fmt.Sprintf("GoroutineScope (%d)", i))
 			vars, err := scope.LocalVariables(pnormalLoadConfig)
@@ -1354,6 +1359,9 @@ func TestCallFunction(t *testing.T) {
 		// Issue 3176
 		{`ref.String()[0]`, []string{`:byte:98`}, nil, 1},
 		{`ref.String()[20]`, nil, errors.New("index out of bounds"), 1},
+
+		// Issue 4136
+		{`nilptrtostruct.VRcvr(0)`, []string{}, errors.New("nil pointer dereference"), 0},
 	}
 
 	var testcases112 = []testCaseCallFunction{
@@ -1576,6 +1584,10 @@ func testCallFunctionIntl(t *testing.T, grp *proc.TargetGroup, p *proc.Target, t
 }
 
 func TestIssue4051(t *testing.T) {
+	if testBackend == "rr" {
+		t.Skip("Skipping TestIssue4051 for rr backend due to Go runtime changes in newer versions")
+	}
+
 	protest.MustSupportFunctionCalls(t, testBackend)
 	protest.AllowRecording(t)
 	withTestProcess("issue4051", t, func(p *proc.Target, grp *proc.TargetGroup, fixture protest.Fixture) {
@@ -1593,11 +1605,11 @@ func TestIssue4051(t *testing.T) {
 
 		err = grp.Continue()
 		assertNoError(err, t, "initial continue to breakpoint failed")
-
 		err = proc.EvalExpressionWithCalls(grp, p.SelectedGoroutine(), `main.Hello("world")`, pnormalLoadConfig, true)
 		if err == nil {
 			t.Fatal("expected error, got nil")
 		}
+
 		expectedError = `expression "main.Hello" is not a function`
 		if err.Error() != expectedError {
 			t.Fatalf("expected error %q, got %q", expectedError, err)
@@ -2053,3 +2065,57 @@ func TestCapturedVarVisibleOnFirstLine(t *testing.T) {
 		}
 	})
 }
+
+// See issue #4116
+func TestEmbeddedStructMethodsAndFieldLookup(t *testing.T) {
+	varTestcases := []varTest{
+		{"v.Model", true, "\"B\"", "", "string", nil},
+		{"v.A.Model", false, "main.(*A).Model", "", "func() string", nil},
+		{"v1.X", false, "main.(*B1).X", "", "func()", nil},
+		{"v1.A1.X", false, "0", "", "int", nil},
+		{"v2.X", false, "main.(*B2).X", "", "func()", nil},
+		{"v2.B2.X", false, "main.(*B2).X", "", "func()", nil},
+		{"v2.B2.A2.X", true, "0", "", "int", nil},
+		{"v2.A2.X", true, "0", "", "int", nil},
+		{"v7.X", false, "main.A7.X", "", "func()", nil},
+		{"x.X", false, "main.(*A7).X", "", "func()", nil},
+		{"x1.X", false, "", "", "", errors.New("x1 (type void) has no member X")},
+		{"x2.X", false, "main.(*A7).X", "", "func()", nil},
+		{"x3.X", false, "main.(*A7).X", "", "func()", nil},
+		{"v9.V.X", false, "main.(*B9).X", "", "func()", nil},
+		{"a.X", false, "main.A8.X", "", "func()", nil},
+		{"b.X", false, "", "", "", errors.New("b has no member X")},
+		{"c.X", false, "main.(*A8).X", "", "func()", nil},
+		{"d.X", true, "1", "", "int", nil},
+		{"v3.X", false, "main.(*B3).X", "", "func()", nil},
+		{"v3.B3.X", false, "main.(*B3).X", "", "func()", nil},
+		{"v4.X", false, "main.(*A4).X", "", "func()", nil},
+		{"v4.TestX.X", false, "main.(*A4).X", "", "func()", nil},
+		{"v5.X", true, "0", "", "int", nil},
+		{"v5.B5.X", false, "main.main.func1", "", "func() string", nil},
+		{"v5.B5.A5.X", false, "main.main.func1", "", "func() string", nil},
+		{"v5.A5.X", false, "main.main.func1", "", "func() string", nil},
+		{"*(ch.buf)", true, "[0]int []", "", "[0]int", nil},
+		{"*(v6.Chan.buf)", true, "[0]struct struct {} []", "", "[0]struct struct {}", nil},
+		{"*(v6.buf)", false, "", "", "", errors.New("v6 has no member buf")},
+	}
+
+	withTestProcess("issue4116", t, func(p *proc.Target, grp *proc.TargetGroup, fixture protest.Fixture) {
+		assertNoError(grp.Continue(), t, "Continue()")
+
+		for _, tc := range varTestcases {
+			variable, err := evalVariableWithCfg(p, tc.name, pnormalLoadConfig)
+			if tc.err == nil {
+				assertNoError(err, t, "EvalVariable() returned an error")
+				assertVariable(t, variable, tc)
+			} else {
+				if err == nil {
+					t.Fatalf("Expected error %s, got no error: %s\n", tc.err.Error(), api.ConvertVar(variable).SinglelineString())
+				}
+				if tc.err.Error() != err.Error() {
+					t.Fatalf("Unexpected error. Expected %s got %s", tc.err.Error(), err.Error())
+				}
+			}
+		}
+	})
+}
